"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const Ajv = require("ajv");
const util_1 = require("./util");
const schema_1 = require("./schema");
class Command {
    constructor(options) {
        this.options = options;
        this.configPath =
            options.configPath || path.join(process.cwd(), 'projects.yml');
        this.config = this.getCommandConfig();
        this.baseDir = this.config.baseDir = options.baseDir || options.targetDir || this.config.baseDir;
    }
    eachProject(fn, options = {}) {
        let value = 0;
        const { projects } = this.config;
        const filteredProjects = options.filter
            ? projects.filter(options.filter)
            : projects;
        for (const project of filteredProjects) {
            value++;
            const projectDir = path.join(this.baseDir, project.package);
            fn(projectDir, project, {
                ...this.options,
                countOf: [value, filteredProjects.length],
            });
        }
    }
    getCommandConfig() {
        if (!fs.existsSync(this.configPath)) {
            throw Error(`config file doesnt exist:\n ${this.configPath}.\n\nPlease use -c or --config-path to specify a path to a config file, or create one.`);
        }
        const config = util_1.getConfig(this.configPath);
        return this.validateConfig(config);
    }
    validateConfig(config) {
        const ajv = new Ajv({ useDefaults: true });
        const ajvValidator = ajv.compile(schema_1.configSchema);
        const valid = ajvValidator(config);
        if (!valid) {
            util_1.logger.error(JSON.stringify(ajvValidator.errors, null, 2));
            throw Error(`Invalid configuration file:`);
        }
        // return the config with defaults added
        return config;
    }
}
exports.Command = Command;
