"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meow = require("meow");
const chalk_1 = require("chalk");
const util_1 = require("./util");
const util_2 = require("../util");
function runCLI(argv) {
    try {
        const [, , commandArg] = argv;
        if (!commandArg) {
            throw Error(`No command provided. Try: ${util_1.listAvailableCommands()}`);
        }
        if (!util_1.commandMap[commandArg]) {
            throw Error(`Command ${chalk_1.default.whiteBright.bold(commandArg)} does not exist.\nTry: ${util_1.listAvailableCommands()}`);
        }
        const SelectedCmd = util_1.commandMap[commandArg];
        util_2.logger.meta(SelectedCmd.commandName);
        const meowConfig = util_1.getMeowConfig(argv);
        if (SelectedCmd.additionalFlags) {
            meowConfig.flags = Object.assign(meowConfig.flags, SelectedCmd.additionalFlags);
        }
        const meowResult = meow(SelectedCmd.commandHelp, meowConfig);
        if (meowResult.flags.help || !commandArg) {
            util_2.logger.help(SelectedCmd.commandHelp + '\n' + util_1.defaultHelp);
            process.exit(0);
        }
        const Cmd = new SelectedCmd(meowResult.flags);
        Cmd.run();
    }
    catch (err) {
        util_2.logger.error(err);
        process.exit(1);
    }
}
exports.runCLI = runCLI;
