"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const execa = require("execa");
const path = require("path");
const fs = require("fs");
const mkdirp = require("mkdirp");
const Command_1 = require("../Command");
const util_1 = require("../util");
const lib_1 = require("../lib");
class Setup extends Command_1.Command {
    constructor(options) {
        super(options);
        this.setupProject = this.setupProject.bind(this);
    }
    setupWorkingDirectory() {
        const cwd = path.join(this.config.baseDir);
        let isGitRepo = true;
        try {
            execa.sync('git', ['status'], { cwd, preferLocal: false });
        }
        catch (err) {
            isGitRepo = false;
        }
        if (isGitRepo) {
            throw Error(`cannot intialize in a git repository`);
        }
        if (!fs.existsSync(cwd)) {
            util_1.logger.warn(`creating working directory`);
            mkdirp.sync(this.config.baseDir);
        }
        else {
            util_1.logger.info(`setup at existing baseDir ${this.config.baseDir}`);
        }
    }
    setupProject(projectDir, project, opts) {
        if (project.types) {
            lib_1.linkTypes(projectDir, project, this.config);
        }
        lib_1.linkDependencies(projectDir, project, opts);
        lib_1.buildProject(projectDir, project, opts);
        lib_1.linkSelf(projectDir, project, opts);
    }
    run() {
        this.setupWorkingDirectory();
        super.eachProject(lib_1.cloneAndInstall);
        lib_1.cloneTypeDefinitions(this.config.baseDir, this.config.typeDefs);
        super.eachProject(this.setupProject);
    }
}
Setup.commandName = 'setup';
Setup.commandHelp = `
sets up project workspace, clones and installs projects, type definitions,  dbuilds and links dependencies

Usage:
  $ yarn-compose setup
      expects projects.yml by default
  $ yarn-compose setup -c path/to/config.yml
      or, specify a path to a config file

Options:
  --force, -f
    force install
`;
Setup.additionalArgs = {
    force: {
        name: 'force',
        alias: '-f',
        type: 'boolean',
    },
};
exports.Setup = Setup;
